options nosource;           /* Do not show source when running...      */

%let SasAppSvr=SASApp;      /* Set the SAS Application Server name     */

/*---------------------------------------------------------------------*/
/* The grdsvc_enable call will go out to the SAS Metadata Server and   */
/* find the SAS Grid Server definition. A return code of 0 means that  */
/* all signons will use the grid. A non-0 return code means that there */
/* is a problem that should be investigated.                           */
%put;%put Trying to enable the grid for grid server &SasAppSvr...;
%let gerc=%sysfunc(grdsvc_enable(_all_,resource=&SasAppSvr));
%put;%put Grid enable return code=&gerc (0 indicates grid was enabled);

/*---------------------------------------------------------------------*/
/* Define a macro to loop to make sure that the grid nodes have been   */
/* set up correctly.                                                   */
%macro loop;
  %let delayNumSecs = 2;
  %let delayUnits   = 1;
  %let numDone      = 0;

  /*-------------------------------------------------------------------*/
  /* If the grdsvc_enable call worked, try to signon to all nodes      */
  %if (&gerc = 0) %then %do;

    /*-----------------------------------------------------------------*/
    /* The grdsvc_nnodes call will provide the number of grid nodes    */
    /* available in the grid.                                          */
    %let nnodes=%sysfunc(grdsvc_nnodes(resource=&SasAppSvr));
    %put;%put Number of grid nodes=&nnodes;

    /*-----------------------------------------------------------------*/
    /* You can view the progress of the signons using the Grid Manager.*/
    /* Watch for Job Names such as SASGrid:xxxx where xxxx is the value*/
    /* of the sysjobid of this SAS session.                            */
    %put;%put Job Name as it will appear in Grid Manager=SASGrid:&sysjobid;%put;

    %let gridTimeStart = %sysfunc(datetime());

    /*-----------------------------------------------------------------*/
    /* Loop through all possible grid nodes and signon without waiting */
    %do i=1 %to &nnodes;

      %put Signing on to node &i;
      signon grdn&i cmacvar=SignonStatus&i swait=no;

    %end;

    /*-----------------------------------------------------------------*/
    /* Determine elapsed time                                          */
    %let now         = %sysfunc(datetime());
    %let elapsedTime = %sysevalf(&now - &gridTimeStart, integer);
    data _null_;
        call symput('prettyTime', trim(left(put(&elapsedTime,time.))));
    run;
    %put Starting signon processing completed at &prettyTime;
    %put Waiting for signons to finish...;

    /*-----------------------------------------------------------------*/
    /* Wait for signons to complete or fail                            */
    %do %while (&numDone ne &nnodes);

      /*---------------------------------------------------------------*/
      /* Loop through all possible grid nodes                          */
      %do i=1 %to &nnodes;

        /*-------------------------------------------------------------*/
        /* If the signon worked, put out information about the node    */
        %if (&&SignonStatus&i = 0) %then %do;

          %let nodename=%sysfunc(grdsvc_getname(grdn&i));
          %let nodeaddr=%sysfunc(grdsvc_getaddr(grdn&i));
          %put; %put Session &i started on grid node: name, addr ===> &nodename, &nodeaddr;%put;%put;

          %let numDone = %eval(&numDone + 1);
          %let SignonStatus&i = -1;
          %end;

        /*-------------------------------------------------------------*/
        /* If the signon failed, put out a message                     */
        %else %if (&&SignonStatus&i eq 1) %then %do;

          %put;%put Signon failed to node &i;

          %let numDone = %eval(&numDone + 1);
          %let SignonStatus&i = -2;

        %end;

      %end;

      /*---------------------------------------------------------------*/
      /* If we are not complete, wait for a couple seconds             */
      %if (&numDone ne &nnodes) %then %do;
        %syscall sleep(delayNumSecs,delayUnits);
      %end;

    %end;

    /*-----------------------------------------------------------------*/
    /* Determine elapsed time                                          */
    %let now         = %sysfunc(datetime());
    %let elapsedTime = %sysevalf(&now - &gridTimeStart, integer);
    data _null_;
        call symput('prettyTime', trim(left(put(&elapsedTime,time.))));
    run;
    %put Signons completed at &prettyTime;

    /*-----------------------------------------------------------------*/
    /* Stop SAS running on the grid nodes.                             */
    signoff _all_;

    /*-----------------------------------------------------------------*/
    /* Determine elapsed time                                          */
    %let now         = %sysfunc(datetime());
    %let elapsedTime = %sysevalf(&now - &gridTimeStart, integer);
    data _null_;
        call symput('prettyTime', trim(left(put(&elapsedTime,time.))));
    run;
    %put Signoffs completed at &prettyTime;

  %end;

  /*-------------------------------------------------------------------*/
  /* If the grdsvc_enable failed, do not signon to any nodes           */
  %else %do;
    %put;%put Grid not enabled, signon loop cancelled;
    %end;

%mend;

/*---------------------------------------------------------------------*/
/* Invoke the loop macro to issue the signons.                         */
/* Monitor the progress of the signon to the nodes using SAS           */
/* Management Console and the Grid Manager.                            */
%loop;

